@@ -2,9 +2,9 @@
 Verisure session, using verisure app api
 '''
 
-import base64
 import json
 import requests
+import pickle
 from . import urls
 import os
 
@@ -57,7 +57,11 @@ class Session(object):
         self._username = username
         self._password = password
         self._cookieFileName = os.path.expanduser(cookieFileName)
-        self._vid = None
+        self._request_headers = {
+            'APPLICATION_ID': 'PS_PYTHON',
+            'Accept': 'application/json',
+            'Content-Type': 'application/json'}
+        self._request_cookies = None
         self._giid = None
         self.installations = None
 
@@ -76,53 +80,37 @@ class Session(object):
         Login before calling any read or write commands
 
         """
-        if os.path.exists(self._cookieFileName):
-            with open(self._cookieFileName, 'r') as cookieFile:
-                self._vid = cookieFile.read().strip()
 
-            try:
+        # First try with the stored cookies, then the full sequence
+        try:
+            with open(self._cookieFileName, 'rb') as f:
+                self._request_cookies = {'vid': pickle.load(f)['vid']}
                 self._get_installations()
-            except ResponseError:
-                self._vid = None
-                os.remove(self._cookieFileName)
+                return
+        except Exception:
+            self._request_cookies = None
 
-        if self._vid is None:
-            self._create_cookie()
-            with open(self._cookieFileName, 'w') as cookieFile:
-                cookieFile.write(self._vid)
-            self._get_installations()
-
-        self._giid = self.installations[0]['giid']
-
-    def _create_cookie(self):
-        auth = 'Basic {}'.format(
-            base64.b64encode(
-                'CPE/{username}:{password}'.format(
-                    username=self._username,
-                    password=self._password).encode('utf-8')
-            ).decode('utf-8'))
-        response = None
-        for base_url in urls.BASE_URLS:
-            urls.BASE_URL = base_url
+        # The login with stored cookies failed, try to get a new one
+        last_exception = None
+        for login_url in ['https://automation01.verisure.com/auth/login',
+                          'https://automation02.verisure.com/auth/login']:
             try:
                 response = requests.post(
-                    urls.login(),
-                    headers={
-                        'Authorization': auth,
-                        'Accept': 'application/json,'
-                                  'text/javascript, */*; q=0.01',
-                    })
-                if 2 == response.status_code // 100:
-                    break
-                elif 503 == response.status_code:
-                    continue
-                else:
-                    raise ResponseError(response.status_code, response.text)
+                    login_url,
+                    headers={'APPLICATION_ID': 'PS_PYTHON'},
+                    auth=(self._username, self._password))
+                _validate_response(response)
+                with open(self._cookieFileName, 'wb') as f:
+                    pickle.dump(response.cookies, f)
+                self._request_cookies = {'vid': response.cookies['vid']}
+                self._get_installations()
+                return
             except requests.exceptions.RequestException as ex:
                 raise LoginError(ex)
+            except Exception as ex:
+                last_exception = ex
 
-        _validate_response(response)
-        self._vid = json.loads(response.text)['cookie']
+        raise LoginError(last_exception)
 
     def _get_installations(self):
         """ Get information about installations """
@@ -132,11 +120,8 @@ class Session(object):
             try:
                 response = requests.get(
                     urls.get_installations(self._username),
-                    headers={
-                        'Cookie': 'vid={}'.format(self._vid),
-                        'Accept': 'application/json,'
-                                  'text/javascript, */*; q=0.01',
-                    })
+                    headers=self._request_headers,
+                    cookies=self._request_cookies)
                 if 2 == response.status_code // 100:
                     break
                 elif 503 == response.status_code:
@@ -148,6 +133,7 @@ class Session(object):
 
         _validate_response(response)
         self.installations = json.loads(response.text)
+        self._giid = self.installations[0]['giid']
 
     def set_giid(self, giid):
         """ Set installation giid
@@ -163,11 +149,8 @@ class Session(object):
         try:
             response = requests.get(
                 urls.overview(self._giid),
-                headers={
-                    'Accept': 'application/json, text/javascript, */*; q=0.01',
-                    'Accept-Encoding': 'gzip, deflate',
-                    'Content-Type': 'application/json',
-                    'Cookie': 'vid={}'.format(self._vid)})
+                headers=self._request_headers,
+                cookies=self._request_cookies)
         except requests.exceptions.RequestException as ex:
             raise RequestError(ex)
         _validate_response(response)
@@ -184,9 +167,8 @@ class Session(object):
         try:
             response = requests.post(
                 urls.smartplug(self._giid),
-                headers={
-                    'Content-Type': 'application/json',
-                    'Cookie': 'vid={}'.format(self._vid)},
+                headers=self._request_headers,
+                cookies=self._request_cookies,
                 data=json.dumps([{
                     "deviceLabel": device_label,
                     "state": state}]))
@@ -205,10 +187,8 @@ class Session(object):
         try:
             response = requests.put(
                 urls.set_armstate(self._giid),
-                headers={
-                    'Accept': 'application/json, text/javascript, */*; q=0.01',
-                    'Content-Type': 'application/json',
-                    'Cookie': 'vid={}'.format(self._vid)},
+                headers=self._request_headers,
+                cookies=self._request_cookies,
                 data=json.dumps({"code": str(code), "state": state}))
         except requests.exceptions.RequestException as ex:
             raise RequestError(ex)
@@ -225,9 +205,8 @@ class Session(object):
         try:
             response = requests.get(
                 urls.get_armstate_transaction(self._giid, transaction_id),
-                headers={
-                    'Accept': 'application/json, text/javascript, */*; q=0.01',
-                    'Cookie': 'vid={}'.format(self._vid)})
+                headers=self._request_headers,
+                cookies=self._request_cookies)
         except requests.exceptions.RequestException as ex:
             raise RequestError(ex)
         _validate_response(response)
@@ -239,9 +218,8 @@ class Session(object):
         try:
             response = requests.get(
                 urls.get_armstate(self._giid),
-                headers={
-                    'Accept': 'application/json, text/javascript, */*; q=0.01',
-                    'Cookie': 'vid={}'.format(self._vid)})
+                headers=self._request_headers,
+                cookies=self._request_cookies)
         except requests.exceptions.RequestException as ex:
             raise RequestError(ex)
         _validate_response(response)
@@ -261,9 +239,8 @@ class Session(object):
         try:
             response = requests.get(
                 urls.history(self._giid),
-                headers={
-                    'Accept': 'application/json, text/javascript, */*; q=0.01',
-                    'Cookie': 'vid={}'.format(self._vid)},
+                headers=self._request_headers,
+                cookies=self._request_cookies,
                 params={
                     "offset": int(offset),
                     "pagesize": int(pagesize),
@@ -282,9 +259,8 @@ class Session(object):
         try:
             response = requests.get(
                 urls.climate(self._giid),
-                headers={
-                    'Accept': 'application/json, text/javascript, */*; q=0.01',
-                    'Cookie': 'vid={}'.format(self._vid)},
+                headers=self._request_headers,
+                cookies=self._request_cookies,
                 params={
                     "deviceLabel": device_label})
         except requests.exceptions.RequestException as ex:
@@ -298,9 +274,8 @@ class Session(object):
         try:
             response = requests.get(
                 urls.get_lockstate(self._giid),
-                headers={
-                    'Accept': 'application/json, text/javascript, */*; q=0.01',
-                    'Cookie': 'vid={}'.format(self._vid)})
+                headers=self._request_headers,
+                cookies=self._request_cookies)
         except requests.exceptions.RequestException as ex:
             raise RequestError(ex)
         _validate_response(response)
@@ -318,10 +293,8 @@ class Session(object):
         try:
             response = requests.put(
                 urls.set_lockstate(self._giid, device_label, state),
-                headers={
-                    'Accept': 'application/json, text/javascript, */*; q=0.01',
-                    'Content-Type': 'application/json',
-                    'Cookie': 'vid={}'.format(self._vid)},
+                headers=self._request_headers,
+                cookies=self._request_cookies,
                 data=json.dumps({"code": str(code)}))
         except requests.exceptions.RequestException as ex:
             raise RequestError(ex)
@@ -338,9 +311,8 @@ class Session(object):
         try:
             response = requests.get(
                 urls.get_lockstate_transaction(self._giid, transaction_id),
-                headers={
-                    'Accept': 'application/json, text/javascript, */*; q=0.01',
-                    'Cookie': 'vid={}'.format(self._vid)})
+                headers=self._request_headers,
+                cookies=self._request_cookies)
         except requests.exceptions.RequestException as ex:
             raise RequestError(ex)
         _validate_response(response)
@@ -356,9 +328,8 @@ class Session(object):
         try:
             response = requests.get(
                 urls.lockconfig(self._giid, device_label),
-                headers={
-                    'Accept': 'application/json, text/javascript, */*; q=0.01',
-                    'Cookie': 'vid={}'.format(self._vid)})
+                headers=self._request_headers,
+                cookies=self._request_cookies)
         except requests.exceptions.RequestException as ex:
             raise RequestError(ex)
         _validate_response(response)
@@ -385,9 +356,8 @@ class Session(object):
         try:
             response = requests.put(
                 urls.lockconfig(self._giid, device_label),
-                headers={
-                    'Content-Type': 'application/json',
-                    'Cookie': 'vid={}'.format(self._vid)},
+                headers=self._request_headers,
+                cookies=self._request_cookies,
                 data=json.dumps(data))
         except requests.exceptions.RequestException as ex:
             raise RequestError(ex)
@@ -403,9 +373,8 @@ class Session(object):
         try:
             response = requests.post(
                 urls.imagecapture(self._giid, device_label),
-                headers={
-                    'Content-Type': 'application/json',
-                    'Cookie': 'vid={}'.format(self._vid)})
+                headers=self._request_headers,
+                cookies=self._request_cookies)
         except requests.exceptions.RequestException as ex:
             raise RequestError(ex)
         _validate_response(response)
@@ -421,9 +390,8 @@ class Session(object):
         try:
             response = requests.get(
                 urls.get_imageseries(self._giid),
-                headers={
-                    'Accept': 'application/json, text/javascript, */*; q=0.01',
-                    'Cookie': 'vid={}'.format(self._vid)},
+                headers=self._request_headers,
+                cookies=self._request_cookies,
                 params={
                     "numberOfImageSeries": int(number_of_imageseries),
                     "offset": int(offset),
@@ -449,7 +417,9 @@ class Session(object):
             response = requests.get(
                 urls.download_image(self._giid, device_label, image_id),
                 headers={
-                    'Cookie': 'vid={}'.format(self._vid)},
+                    'APPLICATION_ID': 'PS_PYTHON',
+                    'Accept': 'image/jpeg'},
+                cookies=self._request_cookies,
                 stream=True)
         except requests.exceptions.RequestException as ex:
             raise RequestError(ex)
@@ -465,9 +435,8 @@ class Session(object):
         try:
             response = requests.get(
                 urls.get_vacationmode(self._giid),
-                headers={
-                    'Accept': 'application/json, text/javascript, */*; q=0.01',
-                    'Cookie': 'vid={}'.format(self._vid)})
+                headers=self._request_headers,
+                cookies=self._request_cookies)
         except requests.exceptions.RequestException as ex:
             raise RequestError(ex)
         _validate_response(response)
@@ -479,9 +448,8 @@ class Session(object):
         try:
             response = requests.get(
                 urls.door_window(self._giid),
-                headers={
-                    'Accept': 'application/json, text/javascript, */*; q=0.01',
-                    'Cookie': 'vid={}'.format(self._vid)})
+                headers=self._request_headers,
+                cookies=self._request_cookies)
         except requests.exceptions.RequestException as ex:
             raise RequestError(ex)
         _validate_response(response)
@@ -493,9 +461,8 @@ class Session(object):
         try:
             response = requests.post(
                 urls.test_ethernet(self._giid),
-                headers={
-                    'Content-Type': 'application/json',
-                    'Cookie': 'vid={}'.format(self._vid)})
+                headers=self._request_headers,
+                cookies=self._request_cookies)
         except requests.exceptions.RequestException as ex:
             raise RequestError(ex)
         _validate_response(response)
@@ -506,8 +473,8 @@ class Session(object):
         try:
             response = requests.delete(
                 urls.login(),
-                headers={
-                    'Cookie': 'vid={}'.format(self._vid)})
+                headers=self._request_headers,
+                cookies=self._request_cookies)
         except requests.exceptions.RequestException as ex:
             raise RequestError(ex)
         _validate_response(response)
@@ -518,9 +485,8 @@ class Session(object):
         try:
             response = requests.get(
                 urls.get_heatpump_state(self._giid, device_label),
-                headers={
-                    'Accept': 'application/json, text/javascript, */*; q=0.01',
-                    'Cookie': 'vid={}'.format(self._vid)})
+                headers=self._request_headers,
+                cookies=self._request_cookies)
         except requests.exceptions.RequestException as ex:
             raise RequestError(ex)
         _validate_response(response)
@@ -535,10 +501,8 @@ class Session(object):
         try:
             response = requests.put(
                 urls.set_heatpump_state(self._giid, device_label),
-                headers={
-                    'Accept': 'application/json',
-                    'Content-Type': 'application/json',
-                    'Cookie': 'vid={}'.format(self._vid)},
+                headers=self._request_headers,
+                cookies=self._request_cookies,
                 data=json.dumps({'mode': mode}))
         except requests.exceptions.RequestException as ex:
             raise RequestError(ex)
@@ -554,10 +518,8 @@ class Session(object):
         try:
             response = requests.put(
                 urls.set_heatpump_state(self._giid, device_label),
-                headers={
-                    'Accept': 'application/json',
-                    'Content-Type': 'application/json',
-                    'Cookie': 'vid={}'.format(self._vid)},
+                headers=self._request_headers,
+                cookies=self._request_cookies,
                 data=json.dumps({'power': power}))
         except requests.exceptions.RequestException as ex:
             raise RequestError(ex)
@@ -573,10 +535,8 @@ class Session(object):
         try:
             response = requests.put(
                 urls.set_heatpump_state(self._giid, device_label),
-                headers={
-                    'Accept': 'application/json',
-                    'Content-Type': 'application/json',
-                    'Cookie': 'vid={}'.format(self._vid)},
+                headers=self._request_headers,
+                cookies=self._request_cookies,
                 data=json.dumps({'fanSpeed': fan_speed}))
         except requests.exceptions.RequestException as ex:
             raise RequestError(ex)
@@ -592,10 +552,8 @@ class Session(object):
         try:
             response = requests.put(
                 urls.set_heatpump_state(self._giid, device_label),
-                headers={
-                    'Accept': 'application/json',
-                    'Content-Type': 'application/json',
-                    'Cookie': 'vid={}'.format(self._vid)},
+                headers=self._request_headers,
+                cookies=self._request_cookies,
                 data=json.dumps({'targetTemperature': target_temp}))
         except requests.exceptions.RequestException as ex:
             raise RequestError(ex)
@@ -611,10 +569,8 @@ class Session(object):
         try:
             response = requests.put(
                 urls.set_heatpump_feature(self._giid, device_label, feature),
-                headers={
-                    'Accept': 'application/json',
-                    'Content-Type': 'application/json',
-                    'Cookie': 'vid={}'.format(self._vid)})
+                headers=self._request_headers,
+                cookies=self._request_cookies)
         except requests.exceptions.RequestException as ex:
             raise RequestError(ex)
         _validate_response(response)
@@ -630,10 +586,8 @@ class Session(object):
         try:
             response = requests.put(
                 urls.set_heatpump_state(self._giid, device_label),
-                headers={
-                    'Accept': 'application/json',
-                    'Content-Type': 'application/json',
-                    'Cookie': 'vid={}'.format(self._vid)},
+                headers=self._request_headers,
+                cookies=self._request_cookies,
                 data=json.dumps({'airSwingDirection':
                                 {"vertical": airswingdirection}}))
         except requests.exceptions.RequestException as ex:
@@ -647,11 +601,8 @@ class Session(object):
         try:
             response = requests.get(
                 urls.get_firmware_status(self._giid),
-                headers={
-                    'Accept': 'application/json, text/javascript, */*; q=0.01',
-                    'Accept-Encoding': 'gzip, deflate',
-                    'Content-Type': 'application/json',
-                    'Cookie': 'vid={}'.format(self._vid)})
+                headers=self._request_headers,
+                cookies=self._request_cookies)
         except requests.exceptions.RequestException as ex:
             raise RequestError(ex)
         _validate_response(response)
